from __future__ import print_function
import sys

from troposphere import Template, Ref, Output, Parameter, Join, GetAtt, FindInMap
from troposphere.route53 import RecordSetType, RecordSet, RecordSetGroup, AliasTarget
from troposphere.s3 import Bucket, PublicRead, BucketOwnerFullControl, WebsiteConfiguration, RedirectAllRequestsTo

import boto3
import botocore


def update(domain):
    t = Template()

    t.add_mapping("RegionMap", {
        "us-east-1" : { "S3hostedzoneID" : "Z3AQBSTGFYJSTF", "websiteendpoint" : "s3-website-us-east-1.amazonaws.com" },
        "us-west-1" : { "S3hostedzoneID" : "Z2F56UZL2M1ACD", "websiteendpoint" : "s3-website-us-west-1.amazonaws.com" },
        "us-west-2" : { "S3hostedzoneID" : "Z3BJ6K6RIION7M", "websiteendpoint" : "s3-website-us-west-2.amazonaws.com" },
        "eu-west-1" : { "S3hostedzoneID" : "Z1BKCTXD74EZPE", "websiteendpoint" : "s3-website-eu-west-1.amazonaws.com" },
        "ap-southeast-1" : { "S3hostedzoneID" : "Z3O0J2DXBE1FTB", "websiteendpoint" : "s3-website-ap-southeast-1.amazonaws.com" },
        "ap-southeast-2" : { "S3hostedzoneID" : "Z1WCIGYICN2BYD", "websiteendpoint" : "s3-website-ap-southeast-2.amazonaws.com" },
        "ap-northeast-1" : { "S3hostedzoneID" : "Z2M4EHUR26P7ZW", "websiteendpoint" : "s3-website-ap-northeast-1.amazonaws.com" },
        "sa-east-1" : { "S3hostedzoneID" : "Z31GFT0UA1I2HV", "websiteendpoint" : "s3-website-sa-east-1.amazonaws.com" }
    })

    hostedzone = t.add_parameter(Parameter(
        "HostedZone",
        Description="The DNS name of an existing Amazon Route 53 hosted zone",
        Type="String",
    ))

    root_bucket = t.add_resource(
        Bucket("RootBucket",
               BucketName=Ref(hostedzone),
               AccessControl=PublicRead,
               WebsiteConfiguration=WebsiteConfiguration(
                   IndexDocument="index.html",
               )
        ))

    www_bucket = t.add_resource(
        Bucket("WWWBucket",
               BucketName=Join('.', ['www', Ref(hostedzone)]),
               AccessControl=PublicRead,
               WebsiteConfiguration=WebsiteConfiguration(
                   RedirectAllRequestsTo=RedirectAllRequestsTo(
                       HostName=Ref(root_bucket)
                   )
        )))

    record = t.add_resource(RecordSetGroup(
        'RecordSetGroup',
        HostedZoneName=Join("", [Ref(hostedzone), "."]),
        RecordSets=[
            RecordSet(
                Name=Ref(hostedzone),
                Type='A',
                AliasTarget=AliasTarget(
                    hostedzoneid=FindInMap('RegionMap', Ref('AWS::Region'), 'S3hostedzoneID'),
                    dnsname=FindInMap('RegionMap', Ref('AWS::Region'), 'websiteendpoint'),
                )
            ),
            RecordSet(
                Name=Join('.', ['www', Ref(hostedzone)]),
                Type='CNAME',
                TTL='900',
                ResourceRecords=[
                    Join('.', ['www', Ref(hostedzone), FindInMap('RegionMap', Ref('AWS::Region'), 'websiteendpoint')])
                ]
            ),
        ]
    ))

    stack_name = domain.replace('.', '')

    client = boto3.client('cloudformation', region_name='eu-west-1')

    try:
        response = client.describe_stacks(
            StackName=stack_name,
        )
    except botocore.exceptions.ClientError:
        client.create_stack(
            StackName=stack_name,
            TemplateBody=t.to_json(),
            Parameters=[
                {'ParameterKey': 'HostedZone', 'ParameterValue': domain},
            ],
        )
        waiter = client.get_waiter('stack_create_complete')
        waiter.wait(StackName=stack_name)
        response = client.describe_stacks(
            StackName=stack_name,
        )

    try:
        client.update_stack(StackName=stack_name, TemplateBody=t.to_json(), Parameters=[
            {'ParameterKey': 'HostedZone', 'ParameterValue': domain},
        ])
        waiter = client.get_waiter('stack_update_complete')
        waiter.wait(StackName=stack_name)
        response = client.describe_stacks(
            StackName=stack_name,
    )
    except Exception as e:
        print(e, file=sys.stderr)
        pass



import pytest
import httpretty
import json
import re
from exosphere import stacks
try:
    from urllib.parse import urlparse, parse_qs
except ImportError:
    from urlparse import urlparse, parse_qs


@pytest.fixture(autouse=True, scope='function')
def mock_responses(request):
    os.environ['AWS_ACCESS_KEY_ID'] = creds['accessKeyId']
    os.environ['AWS_SECRET_ACCESS_KEY'] = creds['secretAccessKey']

    def callback(http_request, uri, headers):
        httpretty.disable()
        response = testypie.get_response(uri, http_request.headers)
        headers.update({key.lower(): value for key, value in
                        response['headers'].iteritems()})
        httpretty.enable()
        return response['code'], headers, response['body'].encode('utf-8')

    def cloudformation(request, uri, headers):
        q = parse_qs(request.querystring or request.body.decode('utf-8'))
        print(q)
        return (200, {}, {
            'GET': {},
            'POST': {
                'DescribeStacks': """<DescribeStacksResponse xmlns="http://cloudformation.amazonaws.com/doc/2010-05-15/">
  <DescribeStacksResult>
    <Stacks>
      <member>
        <StackName>MyStack</StackName>
        <StackId>arn:aws:cloudformation:us-east-1:123456789:stack/MyStack/aaf549a0-a413-11df-adb3-5081b3858e83</StackId>
        <CreationTime>2010-07-27T22:28:28Z</CreationTime>
        <StackStatus>CREATE_COMPLETE</StackStatus>
        <DisableRollback>false</DisableRollback>
        <Outputs>
          <member>
            <OutputKey>StartPage</OutputKey>
            <OutputValue>http://my-load-balancer.amazonaws.com:80/index.html</OutputValue>
          </member>
        </Outputs>
      </member>
    </Stacks>
  </DescribeStacksResult>
  <ResponseMetadata>
    <RequestId>b9b4b068-3a41-11e5-94eb-example</RequestId>
  </ResponseMetadata>
</DescribeStacksResponse>""",
            },
        }[request.method][q.get('Action', ['default'])[0]])
    
    httpretty.register_uri(
        httpretty.GET,
        re.compile('https://cloudformation.eu-west-1.amazonaws.com.*'),
        body=cloudformation,
    )

    httpretty.register_uri(
        httpretty.POST,
        re.compile('https://cloudformation.eu-west-1.amazonaws.com.*'),
        body=cloudformation,
    )

    httpretty.enable()

    request.addfinalizer(httpretty.disable)


def test_create_stack():
    stacks.get('static_site').update('example.com')



#!/usr/bin/env python

import os
from setuptools import setup
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]
    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.tox_args = None
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import tox
        import shlex
        args = self.tox_args
        if args:
            args = shlex.split(self.tox_args)
        errno = tox.cmdline(args=args)
        sys.exit(errno)


setup(
    name="exosphere",
    version="0.0.0",
    author='The Launch Ninja',
    author_email='exosphere@thelaunch.ninja',
    description='Pre-built, useful Cloudformation stacks and commands to work with them.',
    url='http://github.com/bbc/exosphere',
    install_requires=['troposphere', 'boto3', 'clize'],
    packages=['exosphere', 'exosphere.stacks'],
    entry_points={
        'console_scripts': [
            'exosphere = exosphere.cli:main',
        ],
    },
    tests_require=['pytest', 'pytest-cov', 'tox', 'virtualenv', 'httpretty'],
    cmdclass = {'test': Tox},
)
